import requests
import crypto
from account import Account
from rest import Rest

class Client(object):
    def __init__(self, key=None, url=None):
        self.testnet_base_url = "https://live.coinpit.me/api/v1"
        self.livenet_base_url = "https://live.coinpit.io/api/v1"
        self.base_url_map = {0: self.livenet_base_url, 111: self.testnet_base_url}
        self.private_key = key
        self.account = None
        self.rest = None

        self.all = None
        self.contract = None

        if self.private_key is None:
            return
        self.network_code = crypto.get_network_code(self.private_key)
        self.base_url = url if url is not None else self.base_url_map[self.network_code]
        self.rest = Rest(self.base_url)

    def get_server_pubkey(self):
        if self.account is not None:
            return
        if self.private_key is None:
            raise ValueError('Private key needed for protected endpoints')
        user_pub_key = crypto.get_pub_key(self.private_key)
        auth_info = requests.get(self.base_url + "/auth/" + user_pub_key, headers={'Accept': 'application/json'}).json()
        if isinstance(auth_info, dict) and 'error' in auth_info.keys() is not None:
            raise RuntimeError('Error getting public key: ' + auth_info['error'])
        return auth_info['serverPublicKey']

    def connect(self):
        server_pub_key = self.get_server_pubkey()
        self.account = Account(self.private_key, server_pub_key)
        self.rest = Rest(self.base_url, self.account)


    def info(self):
        return self.rest.server_call("/all/info")

    def get_account(self):
        return self.rest.auth_server_call("GET", "/account")

    # def patch_orders(self, patch_spec):
    #     orders.patch(self, patch_spec)
    #
    # def cancel_orders(self, cancel_spec):
    #     orders.cancel(self, cancel_spec)
    #
    # def cancel_all_orders(self):
    #     orders.cancel_all(self)
    #
    # def update_orders(self, update_spec):
    #     orders.update(self, update_spec)
    #
    # def create_orders(self, create_spec):
    #     orders.create(self, create_spec)
    #
    # def get_open_orders(self, get_open_spec):
    #     orders.get_open(self, get_open_spec)
    #
    # def get_closed_orders(self, get_closed_spec):
    #     orders.get_closed(self, get_closed_spec)
    #
    # def get_cancelled_orders(self, get_cancelled_spec):
    #     orders.get_cancelled(self, get_cancelled_spec)

    def get_orders(self, instrument, status="open", after=None):
        uri = "/contract/" + instrument + "/order/" + status + ("" if after is None else "?after=" + after)
        result = self.rest.auth_server_call("GET", uri)
        return result



import crypto
import json
import re
import requests
import time


class Rest(object):

    def __init__(self, base_url, account=None):
        self.base_url = base_url
        self.host = re.split('[:/]*', self.base_url)[1]
        self.account = account
        self.methods = {}
        self.methods['GET']     = requests.get
        self.methods['POST']    = requests.post
        self.methods['PUT']     = requests.put
        self.methods['DELETE']  = requests.delete
        self.methods['PATCH']   = requests.patch
        self.methods['OPTIONS'] = requests.options

    def get(self, url):
        if url.startswith("/all/") or url.startswith("/auth/"):
            return self.server_call(url)
        return self.auth_server_call("GET", url)

    def server_call(self, url, headers={'Accept': 'application/json'}):
        try:
            return requests.get(self.base_url + url, headers=headers)
        except Exception as err:
            print "Error on REST call {} \n {}".format(self.base_url + url, err)

    def sparse_json(self, body=None):
        if body is None:
            return None
        result = json.dumps(body, separators=(',', ':'))
        return None if result == '{}' else result

    def canonical(self, body=None):
       return None if body is None else json.loads(body)

    def auth_server_call(self, method, url, body=None):
        assert self.account is not None, "Call to server requiring auth needs account"
        try:
            canonical_body = self.canonical(body)
            sparse_body    = self.sparse_json(canonical_body)
            headers        = self.get_headers(method, url, sparse_body)
            method         = self.methods[method]

            return method(url=self.base_url + url, json=canonical_body, headers=headers)
        except Exception as err:
            print "Error on REST call {} \n {}".format(self.base_url + url, err)

    def get_headers(self, method, url, body):
        nonce = str(long(time.time() * 1000))
        headers = crypto.get_headers(self.account.user_id, self.account.shared_secret, nonce, method, url, body)
        headers['Host'] = self.host
        return headers



import crypto


class Account(object):
    def __init__(self, private_key, server_public_key):
        self.private_key = private_key
        self.public_key = crypto.get_pub_key(self.private_key)
        self.network_code = crypto.get_network_code(self.private_key)
        self.user_id = crypto.get_user_id(self.public_key, self.network_code)
        self.server_pub_key = server_public_key
        self.shared_secret = crypto.get_shared_secret(self.server_pub_key, self.private_key)



#!/usr/bin/env python
import pycoinpit
import atexit
import sys, getopt
import json

try:
    import readline
except ImportError:
    pass
else:
    import rlcompleter
    readline.parse_and_bind('set editing-mode emacs')

verbose = False
pretty  = False
keyfile = None
url     = None

methods            = {}
methods['GET']     = True
methods['POST']    = True
methods['PUT']     = True
methods['DELETE']  = True
methods['PATCH']   = True
methods['OPTIONS'] = True

def quit_gracefully():
    print "\n"

atexit.register(quit_gracefully)

def usage():
  print "Usage: ", sys.argv[0], " -k keyfile [-v] [-u url]"
  print "    keyfile: Your json file to access coinpit app"
  print "         -v: Verbose"
  print "     -u url: Use alternate url"
  sys.exit(2)

def dump_headers(headers):
    for key in headers.keys():
        print "{}: {}".format(key, headers[key])
    print "==================================="

try:
    opts, args = getopt.getopt(sys.argv[1:], "k:u:vp", ["keyfile=", "url=", "verbose", "pretty"])
except getopt.GetoptError as err:
    print "Error parsing arguments", str(err)
    usage()

for opt, arg in opts:
    if opt == '-h':
        usage()
    elif opt in ("-k", "--keyfile"):
        keyfile = arg
    elif opt in ("-u", "--url"):
        url = arg
    elif opt in ("-v", "--verbose"):
        verbose = True
    elif opt in ("-p", "--pretty"):
        pretty = True

if keyfile == None:
    usage()

if verbose:
    print "Using keyfile: ", keyfile

key = json.load(open(keyfile))
coinpit_me = pycoinpit.client.Client(key['privateKey'], url)
coinpit_me.connect()
user = key['address']
site = coinpit_me.base_url
done = False
prompt = user + ">" if sys.stdout.isatty() and sys.stdin.isatty() else ""

def help():
    print "\nConnected to ", site
    print "Enter REST commands: METHOD path body. Enter quit to exit"
    print "For more information: https://coinpit.io/api"
    print "\nExamples: "
    print '  GET /account'
    print '  POST /order [{"price":1201.2,"side":"buy","quantity":10,"orderType":"LMT","instrument":"BTCUSDW"}]'
    print '  PUT /order [{"price":1201.3,"uuid":"b117ef30-1f50-11e7-b324-e2f410d2f5f7"}]'
    print '  GET /order'
    print '  DELETE /order/b117ef30-1f50-11e7-b324-e2f410d2f5f7'
    print ' '

if verbose:
    help()
while not done:
    try:
        user_input = raw_input(prompt)
        parts = user_input.split(None, 2)
        if(len(parts) == 1):
            command = parts[0].lower()
            if(command == 'quit' or command == 'exit'):
                done = True
            elif(command == 'help'):
                help()
                continue
            elif(command == ''):
                continue
            else:
                print "Unknown command: " + command
            continue
        if(len(parts) < 2):
            print "Unknown command: " + user_input + ": HTTP Method and url (optionally body) expected. Example: GET /account"
            continue
        method_name = parts[0].upper()
        url = parts[1]
        body = None if len(parts) <= 2 else parts[2]
        if(methods[method_name] == None):
            print "Unrecognized command: {}".format(method_name)
            continue
        result = coinpit_me.rest.auth_server_call(method_name, url, body)
        if(verbose):
            canonical = coinpit_me.rest.canonical(body)
            sparse = coinpit_me.rest.sparse_json(canonical)
            print "{} {} HTTP/1.1".format(method_name, site + url)
            dump_headers(coinpit_me.rest.get_headers(method_name, url, sparse))
            print "" if body is None else json.dumps(canonical, indent=4, separators=(',', ':'))
            print result.status_code
            print "==================================="
        if(verbose or pretty):
            print json.dumps(result.json(), indent=4, separators=(',', ':'))
        else:
            print json.dumps(result.json(), separators=(',', ':'))
    except EOFError:
        done = True
    except KeyboardInterrupt:
        print "^C"
        pass
    except Exception as err:
        print "Error on request:\n {}".format(err)



import hashlib
import hmac
import binascii
import pybitcointools
import pyelliptic


def get_headers(user_id, shared_secret, nonce, method, uri, body=None):
    request_string = '{"method":"' + method + '","uri":"' + uri + (
    '",' if (body == None) else '","body":' + body + ',') + '"nonce":' + nonce + '}'
    mac = hmac.new(shared_secret, request_string, hashlib.sha256)
    sig = mac.hexdigest()
    headers = {
        'Authorization': 'HMAC ' + user_id + ':' + sig,
        'Nonce'        : nonce,
        'Accept'       : 'application/json'
    }
    return headers


def get_shared_secret(server_pub_key, private_key):
    user_pub_key = get_pub_key(private_key)
    network_code = get_network_code(private_key)
    uncompressed_user_key = binascii.unhexlify(pybitcointools.decompress(user_pub_key))
    uncompressed_server_key = binascii.unhexlify(pybitcointools.decompress(server_pub_key))
    user_priv_key_bin = binascii.unhexlify(pybitcointools.encode_privkey(private_key, 'hex', network_code))
    user = pyelliptic.ECC(privkey=user_priv_key_bin, pubkey=uncompressed_user_key, curve='secp256k1')
    shared_secret = user.get_ecdh_key(uncompressed_server_key)
    return shared_secret


def get_pub_key(private_key):
    return pybitcointools.privtopub(private_key)


def get_user_id(user_pub_key, network_code):
    return pybitcointools.pubtoaddr(user_pub_key, network_code)


def get_network_code(private_key):
    if (private_key[0] == 'K' or private_key[0] == 'L'):
        network_code = 0
    else:
        network_code = 111
    return network_code



#!/usr/bin/env python
from distutils.core import setup
setup(
  name = 'pycoinpit',
  packages = ['pycoinpit'], # this must be the same as the name above
  scripts = ['coinpit.py'],
  install_requires=[ "pybitcointools", "pyelliptic==1.5.7", "requests" ],
  version = '0.1.3',
  description = 'Coinpit Python Client',
  author = 'Coinpit',
  author_email = 'info@coinpit.io',
  url = 'https://github.com/coinpit/pycoinpit', # use the URL to the github repo
  download_url = 'https://github.com/coinpit/pycoinpit/tarball/0.2',
  keywords = ['coinpit', 'trading', 'python'],
  classifiers = [],
)

# Register package with pypi
#  setup.py register
# Build:
#  python setup.py sdist
# To publish on test:
#  twine upload dist/pycoinpit-0.3.1.tar.gz -r pypitest
# To install test
#  pip install -i https://testpypi.python.org/pypi pycoinpit
#
# To publish production
#  twine upload dist/pycoinpit-0.3.1.tar.gz -r pypi
# To install production
#  pip install pycoinpit
